# @Author : Donglai
# @Time   : 7/12/2020 2:36 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com
import os

CONFIG ={'local_data_dir':'ace_data/',
         'real_data_dir':'real_data/',
         'remote_data_dir':'ftp://ftp.swpc.noaa.gov/pub/lists/ace/'}

