# @Author : Donglai
# @Time   : 7/20/2020 3:55 PM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com
import os


CONFIG ={'local_data_dir':'dst_data/',
         'real_data_dir':'real_data/',
         'remote_data_dir': 'https://unifieddatalibrary.com',
         'services': '/sfm/rest'}